# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/getData.py $
# Last committed:     $Revision: 999449 $; $Author: svcdms $; $Date: 2021-03-17 12:40:26 +0100 (wo, 17 mrt 2021) $
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 11 14:58:35 2019

@author: hove
"""

from pyproj import Proj, transform
from datetime import datetime

from ReadMetaDataAdjusted_NoLog import ReadMetaDataAdjusted

#%%

def get_parameters(instrument):
    
    # ruw ongevalideerd filter in FEWS
    frameworkRO = {'ALT': ['ALTdist'],
                 'STB': ['EtroLevel'],
                 'MPP': ['C_0', 'Chl_0', 'OPAQ_0', 'Ph_0', 'T_0', 'O2_0_mgl', 'O2_0_perc'],
                 'ADC': ['Heading_0', 'Pitch_0', 'Roll_0', 'T_0', 'S_0_sound', 'status_Bit'],
                 'ADV': ['S_0_east', 'S_0_north', 'S_0_up', 'Amp1_0', 'Amp2_0', 'Amp3_0', 'Corr1_0', 'Corr2_0', 'Corr3_0', 
                         'V_0_battery', 'S_0_sound', 'Heading_0', 'Pitch_0', 'Roll_0', 'T_0', 'ErrCode_0', 'Status_0'],
                 'AQD': ['Heading_0', 'Pitch_0', 'Roll_0', 'V_0_battery', 'S_0_sound', 'T_0']
                 }
    
    # ruw gevalideerd filter in FEWS
    frameworkRV = {'ALT': [],
                 'STB': [],
                 'MPP': [],
                 'ADC': [],
                 'ADV': ['D_0_water', 'H_w_0'],
                 'AQD': ['D_0_water', 'H_w_0']
                 }
    
    # parameters standaard data voor Aquo / netcdf    
    frameworkNC = {'ALT': ['H_b_0'],
                 'STB': ['H_w_0'],
                 'MPP': frameworkRO['MPP'],
                 'ADC': frameworkRO['ADC'],
                 'ADV': frameworkRO['ADV']+frameworkRV['ADV'],
                 'AQD': frameworkRO['AQD']+frameworkRV['AQD']
                 }
    
    parametersRO = frameworkRO[instrument] # import from ruw_ongevalideerd
    parametersRV = frameworkRV[instrument] # import from ruw_gevalideerd
    parametersNC = frameworkNC[instrument] # export to netcdf
    
    return parametersRO, parametersRV, parametersNC

#%%
    
def get_parameters_tailored(instrument):
    
    # add parameters for other instruments
    framework = {'STB': ['hmean', 'Hm0', 'Tp', 'Tm01', 'Tm02', 'Tmm10', 'vy'],
                 'ADV': ['hmean', 'Hm0', 'Tp', 'Tm01', 'Tm02', 'Tmm10', 'theta0', 'dspr', 'vy', 'S'],
                 'AQD': ['hmean', 'Hm0', 'Tp', 'Tm01', 'Tm02', 'Tmm10', 'vy']
                 }
    
    parameters_tailored = framework[instrument]
    
    return parameters_tailored
    

#%%
    
# ADC and AQD have levels
def get_parameters_levels(instrument, nr):
    
    # ADC: levels counted from 0
    names_Dir = ['Dir_0_n' + '%02d' % (x) for x in range(nr)]
    names_Corr = ['Corr_0_n' + '%02d' % (x) for x in range(nr)]
    names_S_east = ['S_0_east_n' + '%02d' % (x) for x in range(nr)]
    names_S_north = ['S_0_north_n' + '%02d' % (x) for x in range(nr)]
    names_S_up = ['S_0_up_n' + '%02d' % (x) for x in range(nr)]
    names_S_err = ['S_0_err_n' + '%02d' % (x) for x in range(nr)]
    names_Echo1 = ['E1_0_n' + '%02d' % (x) for x in range(nr)]
    names_Echo2 = ['E2_0_n' + '%02d' % (x) for x in range(nr)]
    names_Echo3 = ['E3_0_n' + '%02d' % (x) for x in range(nr)]
    names_Echo4 = ['E4_0_n' + '%02d' % (x) for x in range(nr)] 
    
    # AQD: note this is x+1 instead of x !!! levels counted from 1
    names_Amp1 = ['Amp1_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_Amp2 = ['Amp2_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_Amp3 = ['Amp3_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_Corr1 = ['Corr1_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_Corr2 = ['Corr2_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_Corr3 = ['Corr3_0_n' + '%02d' % (x+1) for x in range(nr)]
    names_S_east_AQD = ['S_0_east_n' + '%02d' % (x+1) for x in range(nr)]
    names_S_north_AQD = ['S_0_north_n' + '%02d' % (x+1) for x in range(nr)]
    names_S_up_AQD = ['S_0_up_n' + '%02d' % (x+1) for x in range(nr)]
    
    # add parameters for other instruments   
    framework_levels = {'ADC': [names_Dir, names_Corr, names_S_east, names_S_north, names_S_up, names_S_err, names_Echo1, names_Echo2, names_Echo3, names_Echo4],
                        'AQD': [names_Amp1, names_Amp2, names_Amp3, names_Corr1, names_Corr2, names_Corr3, names_S_east_AQD, names_S_north_AQD, names_S_up_AQD]
                        }
    
    framework_aquo_levels = {'ADC': ['names_Dir', 'names_Corr', 'names_S_east', 'names_S_north', 'names_S_up', 'names_S_err', 'names_Echo1', 'names_Echo2', 'names_Echo3', 'names_Echo4'],
                             'AQD': ['names_Amp1', 'names_Amp2', 'names_Amp3', 'names_Corr1', 'names_Corr2', 'names_Corr3', 'names_S_east', 'names_S_north', 'names_S_up']
                        }
    
    parameters_levels = framework_levels[instrument]
    parameters_levels = [item for sublist in parameters_levels for item in sublist]
    
    levels_dict = dict(zip(framework_aquo_levels[instrument], framework_levels[instrument]))
    
    return parameters_levels, levels_dict

#%%

def get_parameters_checks(instrument):
    
    # f_administration.count.INSTRUMENT
    frameworkC = {'ALT': ['N_ALTdist'],
                 'STB': ['N_EtroLevel'],
                 'MPP': ['N_T_0','N_OPAQ_0'],
                 'ADC': ['N_S_0_east_n02'],
                 'ADV': ['N_D_0_water','N_S_0_east'],
                 'AQD': ['N_D_0_water','N_S_0_east_n02']
                 }   
    
    parametersC = frameworkC[instrument] # import from ruw_ongevalideerd
    
    return parametersC

#%%

def get_location_list(instrument, startTime):
    
    if startTime < datetime(2019,9,18,0,0,0):
        location = {'ALT': ['FL65_A', 'FL65_B', 'FL65_C', 'FL69_A', 'FL69_B', 'FL69_C'],
                    'STB': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'MPP': ['FL67', 'FL69', 'BUOY1'],
                    'ADC': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'ADV': ['FL65_A', 'FL65_C', 'FL67', 'FL69', 'FL69_A', 'FL69_C'],
                    'AQD': ['FL65_B', 'FL69_B']
                    }
    elif startTime >= datetime(2019,9,18,0,0,0) and startTime < datetime(2020,6,11,0,0,0):
        location = {'ALT': ['FL67_A', 'FL67_B', 'FL67_C', 'FL70_A', 'FL70_B', 'FL70_C'],
                    'STB': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'MPP': ['FL67', 'FL69', 'BUOY1'],
                    'ADC': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'ADV': ['FL67_A', 'FL67_C', 'FL67', 'FL69', 'FL70_A', 'FL70_C', 'BUOY1'],
                    'AQD': ['FL67_B', 'FL70_B']
                    }        
    elif startTime >= datetime(2020,6,11,0,0,0) and startTime < datetime(2020,12,22,0,0,0):
                location = {'ALT': ['FL67_A', 'FL67_B', 'FL67_C', 'FL69_A', 'FL69_B', 'FL69_C'],
                    'STB': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'MPP': ['FL67', 'FL69', 'BUOY1'],
                    'ADC': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'ADV': ['FL67_A', 'FL67_C', 'FL67', 'FL69', 'FL69_A', 'FL69_C', 'BUOY1'],
                    'AQD': ['FL67_B', 'FL69_B']
                    }
    elif startTime >= datetime(2020,12,22,0,0,0) and startTime < datetime(2021,2,25,0,0,0):
                location = {'ALT': ['FL65_A', 'FL65_B', 'FL65_C', 'FL67_A', 'FL67_B', 'FL67_C'],
                    'STB': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'MPP': ['FL67', 'FL69', 'BUOY1'],
                    'ADC': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'ADV': ['FL65_A', 'FL65_C', 'FL67_A', 'FL67_C', 'FL67', 'FL69', 'BUOY1'],
                    'AQD': ['FL65_B', 'FL67_B']
                    }  
    elif startTime >= datetime(2021,2,25,0,0,0):
                location = {'ALT': ['FL67_A', 'FL67_B', 'FL67_C', 'FL68_A', 'FL68_B', 'FL68_C'],
                    'STB': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'MPP': ['FL67', 'BUOY1'],
                    'ADC': ['FL65', 'FL66', 'FL67', 'FL68', 'FL69', 'FL70'],
                    'ADV': ['FL67', 'FL67_A', 'FL67_C', 'FL68_A', 'FL68_C', 'BUOY1'],
                    'AQD': ['FL67_B', 'FL68_B']
                    }             
        
    location_list = location[instrument]
    
    return location_list

#%%    

def get_meta_data(settings, startTime, location_id, instrument):
    
    print("Getting meta data for location {}".format(location_id))
    file_metadata = settings['metadatafile_location']
    ts = [startTime]
    read_meta_data = ReadMetaDataAdjusted(file_metadata, ts, location_id)
    meta_data = read_meta_data.get_meta_data()
    
    X = meta_data[location_id]['X']
    Y = meta_data[location_id]['Y']
    Z = meta_data[location_id]['{}_NAP_Z'.format(instrument)]
    bedlevel = meta_data[location_id]['bodem_NAP_Z']
    tZ = meta_data[location_id]['{}_NAP_Z_date'.format(instrument)]
    SN = meta_data[location_id]['{}_SerialNumber'.format(instrument)]
    
    if instrument=='ADV' or instrument=='AQD':
        drukoffset = meta_data[location_id]['{}_druk_offset'.format(instrument)]
    else:
        drukoffset = -999
    
    return X, Y, Z, bedlevel, tZ, SN, drukoffset

#%%
    
def get_coordinates(location_id, X, Y):
    
    inProj = Proj(init='epsg:7415') # X,Y
    outProj = Proj(init='epsg:4326') # Lon,Lat
    
    lon, lat = transform(inProj, outProj, X, Y)
    
    return lon, lat
